@@ -74,7 +74,7 @@
 #endif
 
 #include <libxfce4util/libxfce4util.h>
-#include <libxfcegui4/libxfcegui4.h>
+#include <libxfce4ui/libxfce4ui.h>
 
 #include <xfconf/xfconf.h>
 
@@ -82,6 +82,9 @@
 #include "xfce-desktop.h"
 #include "xfce-desktop-enum-types.h"
 
+/* disable setting the x background for bug 7442 */
+//#define DISABLE_FOR_BUG7442
+
 struct _XfceDesktopPriv
 {
     GdkScreen *gscreen;
@@ -149,9 +152,8 @@ static gboolean xfce_desktop_expose(GtkWidget *w,
                                     GdkEventExpose *evt);
 static gboolean xfce_desktop_delete_event(GtkWidget *w,
                                           GdkEventAny *evt);
-static void style_set_cb(GtkWidget *w,
-                         GtkStyle *old_style,
-                         gpointer user_data);
+static void xfce_desktop_style_set(GtkWidget *w,
+                                   GtkStyle *old_style);
 
 static void xfce_desktop_connect_backdrop_settings(XfceDesktop *desktop,
                                                    XfceBackdrop *backdrop,
@@ -210,26 +212,13 @@ xfce_desktop_setup_icon_view(XfceDesktop *desktop)
 #ifdef ENABLE_FILE_ICONS
         case XFCE_DESKTOP_ICON_STYLE_FILES:
             {
-                ThunarVfsPath *path;
-                gchar *desktop_path;
+                GFile *file;
+                const gchar *desktop_path;
                 
-#if GLIB_CHECK_VERSION(2, 14, 0)
-                /* glib always returns non-NULL for _DESKTOP */
-                desktop_path = g_strdup(g_get_user_special_dir(G_USER_DIRECTORY_DESKTOP));
-#else
-                desktop_path = xfce_get_homefile("Desktop", NULL);
-#endif
-
-                path = thunar_vfs_path_new(desktop_path, NULL);
-                if(path) {
-                    manager = xfdesktop_file_icon_manager_new(path, 
-                                                              desktop->priv->channel);
-                    thunar_vfs_path_unref(path);
-                } else {
-                    g_critical("Unable to create ThunarVfsPath for '%s'",
-                               desktop_path);
-                }
-                g_free(desktop_path);
+                desktop_path = g_get_user_special_dir(G_USER_DIRECTORY_DESKTOP);
+                file = g_file_new_for_path(desktop_path);
+                manager = xfdesktop_file_icon_manager_new(file, desktop->priv->channel);
+                g_object_unref(file);
             }
             break;
 #endif
@@ -289,6 +278,7 @@ static void
 set_real_root_window_pixmap(GdkScreen *gscreen,
                             GdkPixmap *pmap)
 {
+#ifndef DISABLE_FOR_BUG7442
     Window xid;
     GdkWindow *groot;
     
@@ -310,8 +300,9 @@ set_real_root_window_pixmap(GdkScreen *gscreen,
     /* and set the root window's BG pixmap, because aterm is somewhat lame. */
     gdk_window_set_back_pixmap(groot, pmap, FALSE);
     /* there really should be a standard for this crap... */
-    
+
     gdk_error_trap_pop();
+#endif
 }
 
 static void
@@ -347,14 +338,29 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
     pix = xfce_backdrop_get_pixbuf(backdrop);
     if(!pix)
         return;
-    
-    if(desktop->priv->nbackdrops == 1) {
-        /* single monitor */
-        rect.x = rect.y = 0;
+
+    if(desktop->priv->xinerama_stretch) {
+        GdkRectangle monitor_rect;
+
+        gdk_screen_get_monitor_geometry(gscreen, 0, &rect);
+
+        /* Get the lowest x and y value for all the monitors in
+         * case none of them start at 0,0 for whatever reason.
+         */
+        for(i = 1; i < (guint)gdk_screen_get_n_monitors(gscreen); i++) {
+            gdk_screen_get_monitor_geometry(gscreen, i, &monitor_rect);
+
+            if(monitor_rect.x < rect.x)
+                rect.x = monitor_rect.x;
+            if(monitor_rect.y < rect.y)
+                rect.y = monitor_rect.y;
+        }
+
         rect.width = gdk_screen_get_width(gscreen);
         rect.height = gdk_screen_get_height(gscreen);
-    } else
+    } else {
         gdk_screen_get_monitor_geometry(gscreen, monitor, &rect);
+    }
 
     gdk_draw_pixbuf(GDK_DRAWABLE(pmap), GTK_WIDGET(desktop)->style->black_gc,
                     pix, 0, 0, rect.x, rect.y,
@@ -374,6 +380,28 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
     set_real_root_window_pixmap(gscreen, pmap);
 }
 
+static void
+backdrop_cycle_cb(XfceBackdrop *backdrop, gpointer user_data)
+{
+    const gchar* backdrop_list;
+
+    g_return_if_fail(XFCE_IS_BACKDROP(backdrop));
+
+    backdrop_list = xfce_backdrop_get_list(backdrop);
+
+    if(xfdesktop_backdrop_list_is_valid(backdrop_list)) {
+        gchar *backdrop_file;
+        GError *error = NULL;
+
+        backdrop_file = xfdesktop_backdrop_list_choose_random(backdrop_list,
+                                                              &error);
+
+        xfce_backdrop_set_image_filename(backdrop, backdrop_file);
+        g_free(backdrop_file);
+        backdrop_changed_cb(backdrop, user_data);
+    }
+}
+
 static void
 screen_size_changed_cb(GdkScreen *gscreen, gpointer user_data)
 {
@@ -413,6 +441,14 @@ screen_size_changed_cb(GdkScreen *gscreen, gpointer user_data)
     }
 }
 
+static void
+screen_composited_changed_cb(GdkScreen *gscreen,
+                             gpointer user_data)
+{
+    /* fake a screen size changed, so the background is properly set */
+    screen_size_changed_cb(gscreen, user_data);
+}
+
 static void
 xfce_desktop_monitors_changed(GdkScreen *gscreen,
                               gpointer user_data)
@@ -438,6 +474,9 @@ xfce_desktop_monitors_changed(GdkScreen *gscreen,
                 g_signal_connect(G_OBJECT(desktop->priv->backdrops[0]),
                                  "changed",
                                  G_CALLBACK(backdrop_changed_cb), desktop);
+                g_signal_connect(G_OBJECT(desktop->priv->backdrops[0]),
+                                 "cycle",
+                                 G_CALLBACK(backdrop_cycle_cb), desktop);
             }
             desktop->priv->nbackdrops = 1;
         }
@@ -463,6 +502,10 @@ xfce_desktop_monitors_changed(GdkScreen *gscreen,
                                      "changed",
                                      G_CALLBACK(backdrop_changed_cb),
                                      desktop);
+                    g_signal_connect(G_OBJECT(desktop->priv->backdrops[i]),
+                                     "cycle",
+                                     G_CALLBACK(backdrop_cycle_cb),
+                                     desktop);
                 }
             }
             desktop->priv->nbackdrops = n_monitors;
@@ -488,6 +531,13 @@ screen_set_selection(XfceDesktop *desktop)
     selection_atom = XInternAtom(GDK_DISPLAY(), selection_name, False);
     manager_atom = XInternAtom(GDK_DISPLAY(), "MANAGER", False);
 
+    /* the previous check in src/main.c occurs too early, so workaround by
+     * adding this one. */
+   if(XGetSelectionOwner(GDK_DISPLAY(), selection_atom) != None) {
+       g_warning("%s: already running, quitting.", PACKAGE);
+       exit(0);
+   }
+
     XSelectInput(GDK_DISPLAY(), xwin, PropertyChangeMask | ButtonPressMask);
     XSetSelectionOwner(GDK_DISPLAY(), selection_atom, xwin, GDK_CURRENT_TIME);
 
@@ -540,6 +590,7 @@ xfce_desktop_class_init(XfceDesktopClass *klass)
     widget_class->expose_event = xfce_desktop_expose;
     widget_class->delete_event = xfce_desktop_delete_event;
     widget_class->popup_menu = xfce_desktop_popup_menu;
+    widget_class->style_set = xfce_desktop_style_set;
     
     signals[SIG_POPULATE_ROOT_MENU] = g_signal_new("populate-root-menu",
                                                    XFCE_TYPE_DESKTOP,
@@ -620,6 +671,7 @@ xfce_desktop_init(XfceDesktop *desktop)
     
     gtk_window_set_type_hint(GTK_WINDOW(desktop), GDK_WINDOW_TYPE_HINT_DESKTOP);
     gtk_window_set_accept_focus(GTK_WINDOW(desktop), FALSE);
+    gtk_window_set_resizable(GTK_WINDOW(desktop), FALSE);
 }
 
 static void
@@ -629,7 +681,7 @@ xfce_desktop_finalize(GObject *object)
     
     g_object_unref(G_OBJECT(desktop->priv->channel));
     g_free(desktop->priv->property_prefix);
-    
+
     G_OBJECT_CLASS(xfce_desktop_parent_class)->finalize(object);
 }
 
@@ -719,7 +771,6 @@ xfce_desktop_realize(GtkWidget *widget)
     GdkAtom atom;
     gint sw, sh;
     Window xid;
-    GdkDisplay *gdpy;
     GdkWindow *groot;
     
     TRACE("entering");
@@ -749,8 +800,7 @@ xfce_desktop_realize(GtkWidget *widget)
             gdk_atom_intern("_NET_WM_WINDOW_TYPE", FALSE),
             gdk_atom_intern("ATOM", FALSE), 32,
             GDK_PROP_MODE_REPLACE, (guchar *)&atom, 1);
-    
-    gdpy = gdk_screen_get_display(desktop->priv->gscreen);
+
     xid = GDK_WINDOW_XID(GTK_WIDGET(desktop)->window);
     groot = gdk_screen_get_root_window(desktop->priv->gscreen);
     
@@ -768,10 +818,10 @@ xfce_desktop_realize(GtkWidget *widget)
 
     xfce_desktop_monitors_changed(desktop->priv->gscreen, desktop);
     
-    g_signal_connect(G_OBJECT(desktop), "style-set",
-                     G_CALLBACK(style_set_cb), NULL);
     g_signal_connect(G_OBJECT(desktop->priv->gscreen), "size-changed",
             G_CALLBACK(screen_size_changed_cb), desktop);
+    g_signal_connect(G_OBJECT(desktop->priv->gscreen), "composited-changed",
+            G_CALLBACK(screen_composited_changed_cb), desktop);
     
     gtk_widget_add_events(GTK_WIDGET(desktop), GDK_EXPOSURE_MASK);
     
@@ -792,8 +842,6 @@ xfce_desktop_unrealize(GtkWidget *widget)
     
     g_return_if_fail(XFCE_IS_DESKTOP(desktop));
     
-    g_signal_handlers_disconnect_by_func(G_OBJECT(desktop),
-                                         G_CALLBACK(style_set_cb), NULL);
     if(gtk_major_version > 2
        || (gtk_major_version == 2 && gtk_minor_version >= 13))
     {
@@ -812,15 +860,21 @@ xfce_desktop_unrealize(GtkWidget *widget)
     
     g_signal_handlers_disconnect_by_func(G_OBJECT(desktop->priv->gscreen),
             G_CALLBACK(screen_size_changed_cb), desktop);
+    g_signal_handlers_disconnect_by_func(G_OBJECT(desktop->priv->gscreen),
+            G_CALLBACK(screen_composited_changed_cb), desktop);
+
+    gdk_error_trap_push();
     
     groot = gdk_screen_get_root_window(desktop->priv->gscreen);
     gdk_property_delete(groot, gdk_atom_intern("XFCE_DESKTOP_WINDOW", FALSE));
     gdk_property_delete(groot, gdk_atom_intern("NAUTILUS_DESKTOP_WINDOW_ID", FALSE));
+
+#ifndef DISABLE_FOR_BUG7442
     gdk_property_delete(groot, gdk_atom_intern("_XROOTPMAP_ID", FALSE));
     gdk_property_delete(groot, gdk_atom_intern("ESETROOT_PMAP_ID", FALSE));
     gdk_window_set_back_pixmap(groot, NULL, FALSE);
-    gdk_flush();
-    
+#endif
+
     if(desktop->priv->backdrops) {
         for(i = 0; i < desktop->priv->nbackdrops; i++) {
             g_snprintf(property_name, 128, XFDESKTOP_IMAGE_FILE_FMT, i);
@@ -830,7 +884,10 @@ xfce_desktop_unrealize(GtkWidget *widget)
         g_free(desktop->priv->backdrops);
         desktop->priv->backdrops = NULL;
     }
-    
+
+    gdk_flush();
+    gdk_error_trap_pop();
+
     g_object_unref(G_OBJECT(desktop->priv->bg_pixmap));
     desktop->priv->bg_pixmap = NULL;
     
@@ -851,18 +908,23 @@ xfce_desktop_button_press_event(GtkWidget *w,
 {
     guint button = evt->button;
     guint state = evt->state;
-    
+    g_return_val_if_fail(XFCE_IS_DESKTOP(w), FALSE);
+
     if(evt->type == GDK_BUTTON_PRESS) {
-        if(button == 2 || (button == 1 && (state & GDK_SHIFT_MASK)
-                           && (state & GDK_CONTROL_MASK)))
+        if(button == 3 || (button == 1 && (state & GDK_SHIFT_MASK))) {
+#ifdef ENABLE_DESKTOP_ICONS
+            if(XFCE_DESKTOP(w)->priv->icons_style != XFCE_DESKTOP_ICON_STYLE_NONE)
+                return FALSE;
+#endif
+            xfce_desktop_popup_root_menu(XFCE_DESKTOP(w),
+                                         button,
+                                         evt->time);
+        } else if(button == 2 || (button == 1 && (state & GDK_SHIFT_MASK)
+                                  && (state & GDK_CONTROL_MASK)))
         {
             xfce_desktop_popup_secondary_root_menu(XFCE_DESKTOP(w),
                                                    button, evt->time);
             return TRUE;
-        } else if(button == 3 || (button == 1 && (state & GDK_SHIFT_MASK))) {
-            xfce_desktop_popup_root_menu(XFCE_DESKTOP(w),
-                                         button, evt->time);
-            return TRUE;
         }
     }
     
@@ -893,7 +955,6 @@ static gboolean
 xfce_desktop_expose(GtkWidget *w,
                     GdkEventExpose *evt)
 {
-    XfceDesktop *desktop;
     GList *children, *l;
     
     /*TRACE("entering");*/
@@ -901,8 +962,6 @@ xfce_desktop_expose(GtkWidget *w,
     if(evt->count != 0)
         return FALSE;
     
-    desktop = XFCE_DESKTOP(w);
-    
     gdk_window_clear_area(w->window, evt->area.x, evt->area.y,
                           evt->area.width, evt->area.height);
     
@@ -928,14 +987,27 @@ xfce_desktop_delete_event(GtkWidget *w,
 }
 
 static void
-style_set_cb(GtkWidget *w,
-             GtkStyle *old_style,
-             gpointer user_data)
+xfce_desktop_style_set(GtkWidget *w,
+                       GtkStyle *old_style)
 {
     XfceDesktop *desktop = XFCE_DESKTOP(w);
+#ifdef ENABLE_DESKTOP_ICONS
+    gdouble old_font_size;
+#endif
     
-    gdk_window_set_back_pixmap(w->window, desktop->priv->bg_pixmap, FALSE);
+    if(GDK_IS_WINDOW(desktop->priv->bg_pixmap))
+        gdk_window_set_back_pixmap(w->window, desktop->priv->bg_pixmap, FALSE);
     gtk_widget_queue_draw(w);
+
+#ifdef ENABLE_DESKTOP_ICONS
+    old_font_size = desktop->priv->system_font_size;
+    if(xfce_desktop_ensure_system_font_size(desktop) != old_font_size
+       && desktop->priv->icon_view && !desktop->priv->icons_font_size_set)
+    {
+        xfdesktop_icon_view_set_font_size(XFDESKTOP_ICON_VIEW(desktop->priv->icon_view),
+                                          desktop->priv->system_font_size);
+    }
+#endif
 }
 
 static void
@@ -1021,8 +1093,13 @@ xfce_desktop_image_filename_changed(XfconfChannel *channel,
 
             xfce_backdrop_set_image_filename(backdrop, backdrop_file);
             g_free(backdrop_file);
-        } else
+
+            xfce_backdrop_set_list(backdrop, g_strdup(filename));
+        } else {
             xfce_backdrop_set_image_filename(backdrop, filename);
+
+            xfce_backdrop_set_list(backdrop, NULL);
+        }
     }
 }
 
@@ -1074,6 +1151,16 @@ xfce_desktop_connect_backdrop_settings(XfceDesktop *desktop,
     xfconf_g_property_bind(channel, buf, G_TYPE_DOUBLE,
                            G_OBJECT(backdrop), "saturation");
 
+    buf[pp_len] = 0;
+    g_strlcat(buf, "backdrop-cycle-enable", sizeof(buf));
+    xfconf_g_property_bind(channel, buf, G_TYPE_BOOLEAN,
+                           G_OBJECT(backdrop), "backdrop-cycle-enable");
+
+    buf[pp_len] = 0;
+    g_strlcat(buf, "backdrop-cycle-timer", sizeof(buf));
+    xfconf_g_property_bind(channel, buf, G_TYPE_UINT,
+                           G_OBJECT(backdrop), "backdrop-cycle-timer");
+
     /* the image filename could be an image or a backdrop list, so we
      * can't just bind the property directly */
     buf[pp_len] = 0;
@@ -1328,30 +1415,29 @@ xfce_desktop_do_menu_popup(XfceDesktop *desktop,
         screen = gtk_widget_get_screen(GTK_WIDGET(desktop));
     else
         screen = gdk_display_get_default_screen(gdk_display_get_default());
-    
-    menu = gtk_menu_new();
-    gtk_menu_set_screen(GTK_MENU(menu), screen);
-    gtk_widget_show(menu);
-    g_signal_connect_swapped(G_OBJECT(menu), "deactivate",
-                             G_CALLBACK(g_idle_add),
-                             (gpointer)xfce_desktop_menu_destroy_idled);
-    
-    g_signal_emit(G_OBJECT(desktop), populate_signal, 0, menu);
-    
-    /* if nobody populated the menu, don't do anything */
-    menu_children = gtk_container_get_children(GTK_CONTAINER(menu));
-    if(!menu_children) {
-        gtk_widget_destroy(menu);
-        return;
-    }
-    
-    g_list_free(menu_children);
-    
-    gtk_menu_attach_to_widget(GTK_MENU(menu), GTK_WIDGET(desktop), NULL);
-    
+
     if(xfdesktop_popup_grab_available(gdk_screen_get_root_window(screen),
                                       activate_time))
     {
+        menu = gtk_menu_new();
+        gtk_menu_set_screen(GTK_MENU(menu), screen);
+        g_signal_connect_swapped(G_OBJECT(menu), "deactivate",
+                                 G_CALLBACK(g_idle_add),
+                                 (gpointer)xfce_desktop_menu_destroy_idled);
+
+        g_signal_emit(G_OBJECT(desktop), populate_signal, 0, menu);
+
+        /* if nobody populated the menu, don't do anything */
+        menu_children = gtk_container_get_children(GTK_CONTAINER(menu));
+        if(!menu_children) {
+            gtk_widget_destroy(menu);
+            return;
+        }
+
+        g_list_free(menu_children);
+
+        gtk_menu_attach_to_widget(GTK_MENU(menu), GTK_WIDGET(desktop), NULL);
+
         /* bug #3652: for some reason passing the correct button here breaks
          * on some systems but not others.  always pass 0 for now. */
         gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 0,
@@ -1417,3 +1503,14 @@ xfce_desktop_refresh(XfceDesktop *desktop)
     xfce_desktop_setup_icon_view(desktop);
 #endif
 }
+
+void xfce_desktop_arrange_icons(XfceDesktop *desktop)
+{
+    g_return_if_fail(XFCE_IS_DESKTOP(desktop));
+
+#ifdef ENABLE_DESKTOP_ICONS
+    g_return_if_fail(XFDESKTOP_IS_ICON_VIEW(desktop->priv->icon_view));
+
+    xfdesktop_icon_view_sort_icons(XFDESKTOP_ICON_VIEW(desktop->priv->icon_view));
+#endif
+}
